/*
	demo-c.c
		a program demonstrating the use of the Diag
		library in C
		this file is part of the Diag library
		last modified 19 Mar 10 th
*/

#include <stdio.h>
#include "CDiag.h"

/*
Notes:

1) SetMatrix has to be called every time because the
   diagonalization routines overwrite the contents of the
   matrix A.

2) HEigensystem, SEigensystem, and TakagiFactor assume they
   are operating on a symmetric matrix and hence ignore the
   below-diagonal entries.  Similarly, HEigensystem assumes
   an Hermitian matrix and thus real diagonal entries.

3) The macros ToComplex, Re, Im are used below.
   These are defined in CDiag.h and make the program
   independent of whether C or C++ is used.
*/


#define n 3

/* make matrix addressable as in Fortran */
#define A(i,j) A[i-1][j-1]


void SetMatrix(double_complex A[n][n])
{
  int i, j;

/* put your matrix definition here: */
  for( i = 1; i <= n; ++i )
    for( j = 1; j <= n; ++j )
      A(i,j) = ToComplex(i, j);
}


#define printr(name, dr) \
  { int i; \
    printf("\n" name ":\n"); \
    for( i = 0; i < n; ++i ) \
      printf("d(%d) = %10.6f\n", i + 1, dr[i]); }

#define printc(name, dc) \
  { int i; \
    printf("\n" name ":\n"); \
    for( i = 0; i < n; ++i ) \
      printf("d(%d) = %10.6f%+10.6f I\n", i + 1, Re(dc[i]), Im(dc[i])); }


int main()
{
  enum { sort = 1 };
  double_complex A[n][n], U[n][n], V[n][n];
  double_complex dc[n];
  double dr[n];

  SetMatrix(A);
  HEigensystem(n, Matrix(A), dr, Matrix(U), sort);
  printr("HEigensystem", dr);

  SetMatrix(A);
  SEigensystem(n, Matrix(A), dc, Matrix(U), sort);
  printc("SEigensystem", dc);

  SetMatrix(A);
  CEigensystem(n, Matrix(A), dc, Matrix(U), sort);
  printc("CEigensystem", dc);

  SetMatrix(A);
  TakagiFactor(n, Matrix(A), dr, Matrix(U), sort);
  printr("TakagiFactor", dr);

  SetMatrix(A);
  SVD(n,n, Matrix(A), dr, Matrix(U), Matrix(V), sort);
  printr("SVD", dr);

  return 0;
}

