/*
	CDiag.h
		the C prototypes for the Diag routines
		last modified 19 Mar 10 th
*/


#ifndef __CDiag_h__
#define __CDiag_h__

#ifdef __cplusplus

#include <complex>
typedef std::complex<double> double_complex;
#define Re(x) (x).real()
#define Im(x) (x).imag()
#define ToComplex(r,i) double_complex(r, i)

extern "C" {

#else

#include <complex.h>
typedef double complex double_complex;
#define Re(x) creal(x)
#define Im(x) cimag(x)
#define ToComplex(r,i) ((r) + (i)*I)

#endif

void HEigensystem(const int n,
  double_complex *A, const int ldA,
  double *d,
  double_complex *U, const int ldU, const int sort);

void SEigensystem(const int n,
  double_complex *A, const int ldA,
  double_complex *d,
  double_complex *U, const int ldU, const int sort);

void CEigensystem(const int n,
  double_complex *A, const int ldA,
  double_complex *d,
  double_complex *U, const int ldU, const int sort);

void TakagiFactor(const int n,
  double_complex *A, const int ldA,
  double *d,
  double_complex *U, const int ldU, const int sort);

void SVD(const int m, const int n,
  double_complex *A, const int ldA,
  double *d,
  double_complex *V, const int ldV,
  double_complex *W, const int ldW, const int sort);

extern int nsweeps;

#ifdef __cplusplus
}
#endif

#define Matrix(A) (double_complex *)(A),sizeof((A)[0])/sizeof((A)[0][0])

#endif

